@@ -53,7 +53,6 @@ struct load_model_inputs
     const bool use_smartcontext = false;
     const bool use_contextshift = false;
     const bool use_fastforward = false;
-    const int clblast_info = 0;
     const int kcpp_main_gpu = 0;
     const char * vulkan_info = nullptr;
     const int batchsize = 512;
@@ -172,7 +171,6 @@ struct sd_load_model_inputs
 {
     const char * model_filename = nullptr;
     const char * executable_path = nullptr;
-    const int clblast_info = 0;
     const int kcpp_main_gpu = 0;
     const char * vulkan_info = nullptr;
     const int threads = 0;
@@ -193,6 +191,7 @@ struct sd_load_model_inputs
     const float lora_multiplier = 1.0f;
     const int lora_apply_mode = 0;
     const char * photomaker_filename = nullptr;
+    const char * upscaler_filename = nullptr;
     const int img_hard_limit = 0;
     const int img_soft_limit = 0;
     const char * devices_override = nullptr;
@@ -224,6 +223,7 @@ struct sd_generation_inputs
     const bool remove_limits = false;
     const bool circular_x = false;
     const bool circular_y = false;
+    const bool upscale = false;
 };
 struct sd_generation_outputs
 {
@@ -232,6 +232,11 @@ struct sd_generation_outputs
     const char * data = "";
     const char * data_extra = "";
 };
+struct sd_upscale_inputs
+{
+    const char * init_images = "";
+    const int upscaling_resize = 0;
+};
 struct sd_info_outputs
 {
     int status = -1;
@@ -242,7 +247,6 @@ struct whisper_load_model_inputs
 {
     const char * model_filename = nullptr;
     const char * executable_path = nullptr;
-    const int clblast_info = 0;
     const int kcpp_main_gpu = 0;
     const char * vulkan_info = nullptr;
     const char * devices_override = nullptr;
@@ -268,7 +272,6 @@ struct tts_load_model_inputs
     const char * ttc_model_filename = nullptr;
     const char * cts_model_filename = nullptr;
     const char * executable_path = nullptr;
-    const int clblast_info = 0;
     const int kcpp_main_gpu = 0;
     const char * vulkan_info = nullptr;
     const int gpulayers = 0;
@@ -298,7 +301,6 @@ struct embeddings_load_model_inputs
     const int threads = 4;
     const char * model_filename = nullptr;
     const char * executable_path = nullptr;
-    const int clblast_info = 0;
     const int kcpp_main_gpu = 0;
     const char * vulkan_info = nullptr;
     const int gpulayers = 0;
